"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ngAdd = void 0;
const schematics_1 = require("@angular-devkit/schematics");
const tasks_1 = require("@angular-devkit/schematics/tasks");
const ast_utils_1 = require("@schematics/angular/utility/ast-utils");
const change_1 = require("@schematics/angular/utility/change");
const dependencies_1 = require("@schematics/angular/utility/dependencies");
const find_module_1 = require("@schematics/angular/utility/find-module");
const workspace_1 = require("@schematics/angular/utility/workspace");
const schematics_utils_1 = require("../schematics-utils");
const version_1 = require("../version");
/***********************************************************************************/
/* Important note: this schematic depends on non-public API of @schematics/angular */
/***********************************************************************************/
function checkRequiredRouterDependency(host, context) {
    const ngRouter = (0, dependencies_1.getPackageJsonDependency)(host, '@angular/router');
    if (!ngRouter) {
        throw new schematics_1.SchematicsException(`You chose to automatically track page view, but this requires @angular/router as a dependency.\n` +
            `You can run "ng add @angular/router" to add it to your application.`);
    }
    if (ngRouter.type !== dependencies_1.NodeDependencyType.Default) {
        context.logger.warn(`You chose to automatically track page view, but @angular/router is listed as "${ngRouter.type}" dependency.`);
    }
}
function addPackageJsonDependencies(options) {
    return (host, context) => {
        (0, dependencies_1.addPackageJsonDependency)(host, {
            type: dependencies_1.NodeDependencyType.Default,
            name: '@ngx-matomo/tracker',
            version: version_1.version,
        });
        if (options.router) {
            checkRequiredRouterDependency(host, context);
            (0, dependencies_1.addPackageJsonDependency)(host, {
                type: dependencies_1.NodeDependencyType.Default,
                name: '@ngx-matomo/router',
                version: version_1.version,
            });
        }
        context.addTask(new tasks_1.NodePackageInstallTask());
        return host;
    };
}
function buildTrackerConfig(options, context, modulePath) {
    const trackerUrl = (0, schematics_utils_1.escapeLiteral)(options.serverUrl || '');
    const siteId = (0, schematics_utils_1.escapeLiteral)(options.siteId || '');
    const scriptUrl = (0, schematics_utils_1.escapeLiteral)(options.scriptUrl || '');
    const embeddedMode = !!scriptUrl && !trackerUrl && !siteId;
    let config;
    if (embeddedMode) {
        config = `{ scriptUrl: '${scriptUrl}' }`;
    }
    else {
        if (scriptUrl) {
            config = `{ trackerUrl: '${trackerUrl}', siteId: '${siteId}', scriptUrl: '${scriptUrl}' }`;
        }
        else {
            config = `{ trackerUrl: '${trackerUrl}', siteId: '${siteId}' }`;
        }
        if (!trackerUrl || !siteId) {
            context.logger.warn('Configuration properties "siteId" and "trackerUrl" are usually required. ' +
                'You will need to manually update your configuration in "' +
                modulePath +
                "'. ");
        }
    }
    return config;
}
function addImportsToNgModule(options, context) {
    return (host) => {
        const modulePath = options.module;
        if (options.skipImport || !modulePath) {
            return host;
        }
        const source = (0, schematics_utils_1.readIntoSourceFile)(host, modulePath);
        const trackerConfig = buildTrackerConfig(options, context, modulePath);
        const trackerDeclaration = `NgxMatomoTrackerModule.forRoot(${trackerConfig})`;
        const routerDeclaration = 'NgxMatomoRouterModule';
        let changes = (0, ast_utils_1.addImportToModule)(source, modulePath, trackerDeclaration, '@ngx-matomo/tracker');
        if (options.router) {
            changes = changes.concat((0, ast_utils_1.addImportToModule)(source, modulePath, routerDeclaration, '@ngx-matomo/router'));
        }
        const recorder = host.beginUpdate(modulePath);
        for (const change of changes) {
            if (change instanceof change_1.InsertChange) {
                recorder.insertLeft(change.pos, change.toAdd);
            }
        }
        host.commitUpdate(recorder);
        return host;
    };
}
function ngAdd(options) {
    return (host, context) => __awaiter(this, void 0, void 0, function* () {
        if (options.path === undefined) {
            options.path = yield (0, workspace_1.createDefaultPath)(host, options.project);
        }
        options.module = (0, find_module_1.findModuleFromOptions)(host, {
            skipImport: options.skipImport,
            path: options.path,
            module: options.module,
            name: '',
        });
        return (0, schematics_1.chain)([addPackageJsonDependencies(options), addImportsToNgModule(options, context)]);
    });
}
exports.ngAdd = ngAdd;
//# sourceMappingURL=index.js.map